@@ -1,6 +1,6 @@
 /*
  * happydigger - program for cataloging archaeological finds
- * Copyright (C) 2004 Joop Stakenborg <pg4i@amsat.org>
+ * Copyright (C) 2004, 2005 Joop Stakenborg <pg4i@amsat.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,8 @@
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <sqlite.h>
+#include <string.h>
+#include <sqlite3.h>
 #include "callbacks.h"
 #include "support.h"
 #include "db.h"
@@ -30,7 +31,8 @@
 
 extern GtkWidget *window;
 extern preferencestype preferences;
-extern sqlite *db;
+extern sqlite3 *db;
+extern statestype state;
 
 /* TOOLBAR */
 void
@@ -229,9 +231,17 @@ static int list_callback (void *store, int argc, char **argv, char **columnNames
 	GtkTreeIter iter;
 
 	gtk_tree_store_append (store, &iter, NULL);
-	gtk_tree_store_set (store, &iter, 0, argv[0], 1, argv[1], 2, argv[2], 3, argv[3], 
-		4, argv[4], 5, argv[5], 6, argv[6], 7, argv[7], 8, argv[8], 9, argv[9], 
-		10, argv[10], 11, argv[11], 12, argv[12], 13, argv[13], 14, argv[14], 15, argv[15], -1);
+	/* unselected comboboxentries produce a NULL entry */
+	if (argv[3] && strcmp(argv[3], "(NULL)"))
+		gtk_tree_store_set (store, &iter, 3, argv[3], -1);
+	if (argv[4] && strcmp(argv[4], "(NULL)"))
+		gtk_tree_store_set (store, &iter, 4, argv[4], -1);
+	if (argv[5] && strcmp(argv[5], "(NULL)"))
+		gtk_tree_store_set (store, &iter, 5, argv[5], -1);
+	/* rest of the data */
+	gtk_tree_store_set (store, &iter, 0, argv[0], 1, argv[1], 2, argv[2],
+		6, argv[6], 7, argv[7], 8, argv[8], 9, argv[9], 10, argv[10],
+		11, argv[11], 12, argv[12], 13, argv[13], 14, argv[14], 15, argv[15], -1);
 	return 0;
 }
 
@@ -239,13 +249,15 @@ void
 on_listbutton_clicked                  (GtkButton       *button,
                                         gpointer         user_data)
 {
-	GtkWidget *listwindow, *listscrolledwindow, *treeview;
+	GtkWidget *listwindow, *listscrolledwindow, *treeview, *handlebox, *identry;
 	GtkTreeStore *store;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
-	gchar **split, *query, *err;
-	gint i, ret;
-
+	gchar **split, *query, *err, *id;
+	gint i, ret, findnr, pathnr;
+	GObject *selection;
+	GtkTreePath *path;
+	
 	listwindow = create_listwindow ();
 	listscrolledwindow = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(listscrolledwindow),
@@ -257,71 +269,102 @@ on_listbutton_clicked                  (GtkButton       *button,
 		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
 		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
 		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
-  treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL(store));
-  g_object_unref (G_OBJECT (store));
+	treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL(store));
+	g_object_unref (G_OBJECT (store));
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Id"), renderer, "text", 0, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Id"), renderer, "text", 0, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Number"), renderer, "text", 1, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Number"), renderer, "text", 1, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 1);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Description"), renderer, "text", 2, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Description"), renderer, "text", 2, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 2);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Type"), renderer, "text", 3, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Type"), renderer, "text", 3, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 3);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Material"), renderer, "text", 4, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Material"), renderer, "text", 4, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 4);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Period"), renderer, "text", 5, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Period"), renderer, "text", 5, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 5);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Findnotes"), renderer, "text", 6, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Findnotes"), renderer, "text", 6, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 6);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Date"), renderer, "text", 7, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Date"), renderer, "text", 7, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 7);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Value"), renderer, "text", 8, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Value"), renderer, "text", 8, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 8);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Reference"), renderer, "text", 9, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Reference"), renderer, "text", 9, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 9);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Find date"), renderer, "text", 10, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Find date"), renderer, "text", 10, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 10);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-	renderer = gtk_cell_renderer_text_new ();
 
-	column = gtk_tree_view_column_new_with_attributes (_("Location"), renderer, "text", 11, NULL);
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Location"), renderer, "text", 11, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 11);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-	renderer = gtk_cell_renderer_text_new ();
 
-	column = gtk_tree_view_column_new_with_attributes (_("Coordinates"), renderer, "text", 12, NULL);
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Coordinates"), renderer, "text", 12, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 12);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-	renderer = gtk_cell_renderer_text_new ();
 
-	column = gtk_tree_view_column_new_with_attributes (_("Spotnotes"), renderer, "text", 13, NULL);
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Spotnotes"), renderer, "text", 13, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 13);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-	renderer = gtk_cell_renderer_text_new ();
 
-	column = gtk_tree_view_column_new_with_attributes (_("Image1"), renderer, "text", 14, NULL);
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Image1"), renderer, "text", 14, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 14);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-	renderer = gtk_cell_renderer_text_new ();
 
-	column = gtk_tree_view_column_new_with_attributes (_("Image2"), renderer, "text", 15, NULL);
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Image2"), renderer, "text", 15, NULL);
+	gtk_tree_view_column_set_sort_column_id (column, 15);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	/* column with ID is hidden, other depend on preferences */
@@ -331,21 +374,37 @@ on_listbutton_clicked                  (GtkButton       *button,
 	for (i = 0; i < 15; i++)
 	{       
 		column = gtk_tree_view_get_column (GTK_TREE_VIEW(treeview), i + 1);
-		gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(column), atoi(split[i]) ? TRUE : FALSE);
+		gtk_tree_view_column_set_visible
+			(GTK_TREE_VIEW_COLUMN(column), atoi(split[i]) ? TRUE : FALSE);
 	}
 	g_strfreev(split);
-
-  gtk_widget_show (treeview);
-  gtk_container_add (GTK_CONTAINER (listscrolledwindow), treeview);
+	gtk_container_add (GTK_CONTAINER (listscrolledwindow), treeview);
 					
-	query = g_strdup_printf ("SELECT * from finds;");
-	ret = sqlite_exec (db, query, list_callback, store, &err);
+	query = g_strdup_printf ("SELECT * from %s;", state.table);
+	ret = sqlite3_exec (db, query, list_callback, store, &err);
 	if (ret != SQLITE_OK)
 		g_print ("Error on SELECT: %s\n", err);
 	g_free (query);
 
-	gtk_widget_set_sensitive (window, 0);
-	gtk_widget_show (listwindow);
+	/* hide handlebox, so we can't update when the list is visible */
+	handlebox = lookup_widget (window, "handlebox1");
+	gtk_widget_hide (handlebox);
+
+	gtk_widget_show_all (listwindow);
+
+	/* select current record in the list */
+ 	identry = lookup_widget (window, "identry");
+	id = gtk_editable_get_chars (GTK_EDITABLE(identry), 0, -1);
+	findnr = atoi (id);
+	pathnr = findnr - 1;
+	id = g_strdup_printf ("%d", pathnr);
+	path = gtk_tree_path_new_from_string (id);
+	g_free (id);
+	selection = G_OBJECT (gtk_tree_view_get_selection(GTK_TREE_VIEW (treeview)));
+	gtk_tree_selection_select_path (GTK_TREE_SELECTION(selection), path);
+	gtk_tree_path_free (path);
+
+	g_signal_connect (selection, "changed", G_CALLBACK (on_list_select_row), NULL);
 }
 
 void
@@ -406,10 +465,14 @@ on_helpbutton_clicked                  (GtkButton       *button,
                                         gpointer         user_data)
 {
 	GtkWidget *helpwindow, *helptextview;
-	gchar *helpfile, buf[80], *b;
+	gchar *helpfile, *b;
 	FILE *in;
 	GtkTextBuffer *buffer;
-	GtkTextIter iter;
+	GtkTextIter start, end;
+	PangoFontDescription *font_desc;
+	const gchar *encoding;
+	gsize utf8_len, read_len;
+	GError *err;
 
 	helpwindow = create_helpwindow ();
 	helptextview = lookup_widget (helpwindow, "helptextview");
@@ -422,19 +485,28 @@ on_helpbutton_clicked                  (GtkButton       *button,
  */
 	helpfile = g_strdup_printf ("%s%s%s", PACKAGE_DATA_DIR, G_DIR_SEPARATOR_S, _("README"));
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(helptextview));
-	gtk_text_buffer_get_start_iter (buffer, &iter);
+	gtk_text_buffer_get_bounds (buffer, &start, &end);
+	g_get_charset (&encoding);
+
 	b = g_new0 (gchar, 100);
 	in = fopen (helpfile, "r");
 	if (in)
 	{
 		do 
 		{
-			if (fgets (buf, 80, in) == NULL) 
+			if (fgets (b, 80, in) == NULL) 
 				break;
 			else
 			{
-				b = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-				gtk_text_buffer_insert (buffer, &iter, b, -1);
+				err = NULL;
+				g_convert_with_fallback
+(b, strlen(b), "UTF-8", encoding, "?", &read_len, &utf8_len, &err);
+				if (err)
+				{
+					g_print ("%s\n", err->message);
+					g_error_free (err);
+				}
+				gtk_text_buffer_insert (buffer, &start, b, -1);
 			}
 		} 
 		while (!feof (in));
@@ -442,6 +514,9 @@ on_helpbutton_clicked                  (GtkButton       *button,
 	}
 	g_free (b);
 	g_free (helpfile);
+	font_desc = pango_font_description_from_string ("mono");
+	gtk_widget_modify_font (helptextview, font_desc);
+	pango_font_description_free (font_desc);
 	gtk_widget_show (helpwindow);
 }
 
