@@ -1,6 +1,6 @@
 /*
  * happydigger - program for cataloging archaeological finds
- * Copyright (C) 2004 Joop Stakenborg <pg4i@amsat.org>
+ * Copyright (C) 2004, 2005 Joop Stakenborg <pg4i@amsat.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,8 @@ typedef struct
 
 typedef struct
 {
-  gchar *loaddir;
+  gchar *imageloaddir; /* last images loaded from this directory */
+  gchar *table;        /* name of the table used */
 } statestype;
 
 void loadpreferences (void);
